#define _GNU_SOURCE

#include <libguile.h>

#include <string.h>

#include <libintl.h>
#include <locale.h>
#include <errno.h>

SCM
scm_dlgettext(SCM domain, SCM msgid, SCM locale)
{
    SCM_ASSERT(scm_is_string(domain), domain, SCM_ARG1, "dlgettext");
    SCM_ASSERT(scm_is_string(msgid), msgid, SCM_ARG1, "dlgettext");
    SCM_ASSERT(scm_is_string(locale), locale, SCM_ARG1, "dlgettext");

    char *s_domain = scm_to_utf8_stringn(domain, NULL);
    char *s_msgid = scm_to_utf8_stringn(msgid, NULL);
    char *s_locale = scm_to_utf8_stringn(locale, NULL);

    locale_t	new_locale = NULL;
    locale_t	old_locale = NULL;
    const char *s_result = s_msgid;
    SCM  result;

    if ((new_locale = newlocale(LC_CTYPE_MASK|LC_MESSAGES_MASK, s_locale, NULL)) != NULL)
    {
	old_locale = uselocale(new_locale);
	s_result = dgettext(s_domain,s_msgid);
	uselocale(old_locale);

	freelocale(new_locale);
    }

    result = scm_from_utf8_string(s_result);

    free(s_domain);
    free(s_msgid);
    free(s_locale);

    return result;
}

void scm_init_gettext(void)
{
    scm_c_define_gsubr("dlgettext", 3, 0, 0, scm_dlgettext);
}
