#include <pty.h>  /* for openpty and forkpty */
#include <sys/ioctl.h>
#include <unistd.h>

#include <libguile.h>

static SCM sym_master_pty;
static SCM sym_slave_pty;

SCM scm_make_pty()
{
	int master = -1, slave = -1;

	if (openpty(&master, &slave, 0, 0, 0) < 0)
		return SCM_BOOL_F;
	else
		return scm_cons(scm_fdes_to_port(master,"rw",sym_master_pty),
		                scm_fdes_to_port(slave,"rw",sym_slave_pty));
}

SCM scm_set_controlling_terminal(SCM port)
{
	SCM_ASSERT(SCM_PORTP(port), port, SCM_ARG1, "bind-pty");
	int fd = SCM_FPORT_FDES(port);

	if (setsid() < 0)
	    scm_syserror("setsid");
	if (ioctl(fd, (unsigned long) TIOCSCTTY, 0) < 0)
	    scm_syserror("ioctl(TIOCSCTTY)");

	return SCM_UNSPECIFIED;
}

void scm_init_pipe(void)
{
	sym_master_pty = scm_from_utf8_symbol("master pty");
	sym_slave_pty = scm_from_utf8_symbol("slave pty");

	scm_c_define_gsubr("make-pty", 0, 0, 0, scm_make_pty);
	scm_c_define_gsubr("set-controlling-terminal", 1, 0, 0, scm_set_controlling_terminal);
}
